from Gaudi.Configuration import *
import os
  
from GaudiKernel.DataObjectHandleBase import DataObjectHandleBase
from Configurables import ApplicationMgr, EICDataSvc, PodioOutput, GeoSvc
from GaudiKernel import SystemOfUnits as units

detector_name = "topside"
if "JUGGLER_DETECTOR" in os.environ :
  detector_name = str(os.environ["JUGGLER_DETECTOR"])

detector_path = "topside"
if "JUGGLER_DETECTOR_PATH" in os.environ :
  detector_path = str(os.environ["JUGGLER_DETECTOR_PATH"]) 

# todo add checks
input_sim_file = "jug_input.root"
if "JUGGLER_SIM_FILE" in os.environ :
  input_sim_file  = str(os.environ["JUGGLER_SIM_FILE"])
else :
    print(" ERROR : JUGGLER_SIM_FILE not set" )

output_rec_file = "jug_rec.root"
if "JUGGLER_REC_FILE" in os.environ :
  output_rec_file = str(os.environ["JUGGLER_REC_FILE"])
else :
    print(" ERROR : JUGGLER_REC_FILE not set" )


n_events = 100
if "JUGGLER_N_EVENTS" in os.environ :
  n_events = str(os.environ["JUGGLER_N_EVENTS"])

geo_service  = GeoSvc("GeoSvc", detectors=["{}/{}.xml".format(detector_path,detector_name)], OutputLevel=INFO)
podioevent   = EICDataSvc("EventDataSvc", inputs=[input_sim_file], OutputLevel=INFO)

from Configurables import PodioInput
from Configurables import Jug__Base__InputCopier_dd4pod__Geant4ParticleCollection_dd4pod__Geant4ParticleCollection_ as MCCopier
from Configurables import Jug__Base__InputCopier_dd4pod__CalorimeterHitCollection_dd4pod__CalorimeterHitCollection_ as CalCopier



from Configurables import Jug__Digi__CrystalEndcapsDigi as CrystalEndcapsDigi
from Configurables import Jug__Reco__CrystalEndcapsReco as CrystalEndcapsReco
from Configurables import Jug__Reco__CalorimeterIslandCluster as IslandCluster
from Configurables import Jug__Reco__ClusterRecoCoG as RecoCoG

podioinput = PodioInput("PodioReader", collections=["mcparticles","EcalEndcapNHits"], OutputLevel=DEBUG)

## copiers to get around input --> output copy bug. Note the "2" appended to the output collection.
copier    = MCCopier("MCCopier", inputCollection="mcparticles", outputCollection="mcparticles2",OutputLevel=INFO) 
calcopier = CalCopier("CalCopier", inputCollection="EcalEndcapNHits", outputCollection="EcalEndcapNHits2",OutputLevel=INFO) 

emcaldigi = CrystalEndcapsDigi("ecal_digi", 
        inputHitCollection="EcalEndcapNHits", 
        outputHitCollection="RawDigiEcalHits")

emcalreco = CrystalEndcapsReco("ecal_reco", 
        inputHitCollection="RawDigiEcalHits", 
        outputHitCollection="RecoEcalHits",
        minModuleEdep=0.00001*units.MeV)

emcalcluster = IslandCluster("emcal_cluster", 
        inputHitCollection="RecoEcalHits", 
        outputHitCollection="EcalClusterHits",
        minClusterCenterEdep=50.0*units.MeV, 
        groupRange=2.0)

clusterreco = RecoCoG("cluster_reco", 
        inputHitCollection="EcalClusterHits", 
        outputClusterCollection="EcalClusters", 
        logWeightBase=4.2, 
        moduleDimZName="CrystalModule_sz")

out = PodioOutput("out", filename=output_rec_file)

out.outputCommands = ["keep *"]

ApplicationMgr(
    TopAlg = [podioinput, copier, calcopier,
              emcaldigi, emcalreco, emcalcluster, clusterreco, out],
    EvtSel = 'NONE',
    EvtMax   = n_events,
    ExtSvc = [podioevent],
    OutputLevel=INFO
 )
