#!/bin/bash

print_env.sh

function print_the_help {
  echo "USAGE: ${0} -n <nevents> -t <nametag> -p <particle> "
  echo "  OPTIONS: "
  echo "    -n,--nevents     Number of events"
  echo "    -t,--nametag     name tag"
  echo "    -p,--particle    particle type"
  echo "                     allowed types: pion0, pion+, pion-, kaon0, kaon+, kaon-, proton, neutron, electron, positron, photon"
  exit
}

POSITIONAL=()
while [[ $# -gt 0 ]]
do
  key="$1"

  case $key in
    -h|--help)
      shift # past argument
      print_the_help
      ;;
    -t|--nametag)
      nametag="$2"
      shift # past argument
      shift # past value
      ;;
    -p|--particle)
      particle="$2"
      shift # past argument
      shift # past value
      ;;
    -n|--nevents)
      export CB_EMCAL_NUMEV="$2"
      shift # past argument
      shift # past value
      ;;
    *)    # unknown option
      #POSITIONAL+=("$1") # save it in an array for later
      echo "unknown option $1"
      print_the_help
      shift # past argument
      ;;
  esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

# export CB_EMCAL_COMPACT_PATH=${DETECTOR_PATH}/${JUGGLER_DETECTOR}.xml
cp benchmarks/imaging_ecal/test_geo.xml ${DETECTOR_PATH}/.
export CB_EMCAL_COMPACT_PATH=${DETECTOR_PATH}/test_geo.xml

if [[ ! -n  "${CB_EMCAL_NUMEV}" ]] ; then
  export CB_EMCAL_NUMEV=1000
fi

if [[ ! -n  "${CB_EMCAL_ENERGY}" ]] ; then
  export CB_EMCAL_ENERGY=5.0
fi

if [[ ! -n "${CB_EMCAL_SAMP_FRAC}" ]] ; then
  export CB_EMCAL_SAMP_FRAC=0.014
fi

export CB_EMCAL_NAME_TAG="${nametag}"
export CB_EMCAL_GEN_FILE="${CB_EMCAL_NAME_TAG}.hepmc"

export CB_EMCAL_SIM_FILE="sim_${CB_EMCAL_NAME_TAG}.root"
export CB_EMCAL_REC_FILE="rec_${CB_EMCAL_NAME_TAG}.root"

echo "CB_EMCAL_NUMEV = ${CB_EMCAL_NUMEV}"
echo "CB_EMCAL_COMPACT_PATH = ${CB_EMCAL_COMPACT_PATH}"

# Generate the input events
python benchmarks/imaging_ecal/scripts/gen_particles.py ${CB_EMCAL_GEN_FILE} -n ${CB_EMCAL_NUMEV}\
    --angmin 60 --angmax 120 --parray ${CB_EMCAL_ENERGY} --particles="${particle}"
if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running script: generating input events"
  exit 1
fi

ls -lh ${CB_EMCAL_GEN_FILE}

# Run geant4 simulations
npsim --runType batch \
      -v WARNING \
      --part.minimalKineticEnergy "1*TeV" \
      --numberOfEvents ${CB_EMCAL_NUMEV} \
      --compactFile ${CB_EMCAL_COMPACT_PATH} \
      --inputFiles ${CB_EMCAL_GEN_FILE} \
      --outputFile ${CB_EMCAL_SIM_FILE}

if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running npdet"
  exit 1
fi

rootls -t "${CB_EMCAL_SIM_FILE}"

# Directory for plots
mkdir -p results

CB_EMCAL_OPTION_DIR=benchmarks/imaging_ecal/options
# Run Juggler
xenv -x ${JUGGLER_INSTALL_PREFIX}/Juggler.xenv \
    gaudirun.py ${CB_EMCAL_OPTION_DIR}/imaging_2dcluster.py
if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running juggler"
  exit 1
fi

# Plot clusters first
FULL_CAL_SCRIPT_DIR=benchmarks/clustering/scripts
python ${FULL_CAL_SCRIPT_DIR}/cluster_plots.py ${CB_EMCAL_SIM_FILE} ${CB_EMCAL_REC_FILE} -o results \
    --collections "EcalBarrelClusters"

root_filesize=$(stat --format=%s "${CB_EMCAL_REC_FILE}")
if [[ "${CB_EMCAL_NUMEV}" -lt "500" ]] ; then
  # file must be less than 10 MB to upload
  if [[ "${root_filesize}" -lt "10000000" ]] ; then
    cp ${CB_EMCAL_REC_FILE} results/.
  fi
fi

